@@ -1,15 +1,17 @@
 package App::VivaldiUtils;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2019-12-05'; # DATE
+our $DATE = '2019-12-09'; # DATE
 our $DIST = 'App-VivaldiUtils'; # DIST
-our $VERSION = '0.001'; # VERSION
+our $VERSION = '0.002'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
 use warnings;
 use Log::ger;
 
+use App::BrowserUtils ();
+
 our %SPEC;
 
 $SPEC{':package'} = {
@@ -17,111 +19,37 @@ $SPEC{':package'} = {
     summary => 'Utilities related to the Vivaldi browser',
 };
 
-our %argopt_users = (
-    users => {
-        'x.name.is_plural' => 1,
-        'x.name.singular' => 'user',
-        summary => 'Kill Vivaldi processes of certain users only',
-        schema => ['array*', of=>'unix::local_uid*'],
-    },
-);
-
-our %argopt_quiet = (
-    quiet => {
-        schema => 'true*',
-        cmdline_aliases => {q=>{}},
-    },
-);
-
-our %args_common = (
-    %argopt_users,
-);
-
-sub _do_vivaldi {
-    require Proc::Find;
-
-    my ($which, %args) = @_;
-
-    my $procs = Proc::Find::find_proc(
-        detail => 1,
-        filter => sub {
-            my $p = shift;
-
-            if ($args{users} && @{ $args{users} }) {
-                return 0 unless grep { $p->{uid} == $_ } @{ $args{users} };
-            }
-            return 0 unless $p->{fname} =~ /\A(vivaldi-bin)\z/;
-            log_trace "Found PID %d (cmdline=%s, fname=%s, uid=%d)", $p->{pid}, $p->{cmndline}, $p->{fname}, $p->{uid};
-            1;
-        },
-    );
-
-    my @pids = map { $_->{pid} } @$procs;
-
-    if ($which eq 'ps') {
-        return [200, "OK", $procs, {'table.fields'=>[qw/pid uid euid state/]}];
-    } elsif ($which eq 'pause') {
-        kill STOP => @pids;
-        [200, "OK", "", {"func.pids" => \@pids}];
-    } elsif ($which eq 'unpause') {
-        kill CONT => @pids;
-        [200, "OK", "", {"func.pids" => \@pids}];
-    } elsif ($which eq 'terminate') {
-        kill KILL => @pids;
-        [200, "OK", "", {"func.pids" => \@pids}];
-    } elsif ($which eq 'is_paused') {
-        my $num_stopped = 0;
-        my $num_unstopped = 0;
-        my $num_total = 0;
-        for my $proc (@$procs) {
-            $num_total++;
-            if ($proc->{state} eq 'stop') { $num_stopped++ } else { $num_unstopped++ }
-        }
-        my $is_paused = $num_total == 0 ? undef : $num_stopped == $num_total ? 1 : 0;
-        my $msg = $num_total == 0 ? "There are no Vivaldi processes" :
-            $num_stopped   == $num_total ? "Vivaldi is paused (all processes are in stop state)" :
-            $num_unstopped == $num_total ? "Vivaldi is NOT paused (all processes are not in stop state)" :
-            "Vivaldi is NOT paused (some processes are not in stop state)";
-        return [200, "OK", $is_paused, {
-            'cmdline.exit_code' => $is_paused ? 0:1,
-            'cmdline.result' => $args{quiet} ? '' : $msg,
-        }];
-    } else {
-        die "BUG: unknown command";
-    }
-}
-
 $SPEC{ps_vivaldi} = {
     v => 1.1,
     summary => "List Vivaldi processes",
     args => {
-        %args_common,
+        %App::BrowserUtils::args_common,
     },
 };
 sub ps_vivaldi {
-    _do_vivaldi('ps', @_);
+    App::BrowserUtils::_do_browser('ps', 'vivaldi', @_);
 }
 
 $SPEC{pause_vivaldi} = {
     v => 1.1,
     summary => "Pause (kill -STOP) Vivaldi",
     args => {
-        %args_common,
+       %App::BrowserUtils::args_common,
     },
 };
 sub pause_vivaldi {
-    _do_vivaldi('pause', @_);
+    App::BrowserUtils::_do_browser('pause', 'vivaldi', @_);
 }
 
 $SPEC{unpause_vivaldi} = {
     v => 1.1,
     summary => "Unpause (resume, continue, kill -CONT) Vivaldi",
     args => {
-        %args_common,
+        %App::BrowserUtils::args_common,
     },
 };
 sub unpause_vivaldi {
-    _do_vivaldi('unpause', @_);
+    App::BrowserUtils::_do_browser('unpause', 'vivaldi', @_);
 }
 
 $SPEC{vivaldi_is_paused} = {
@@ -133,23 +61,23 @@ Vivaldi is defined as paused if *all* of its processes are in 'stop' state.
 
 _
     args => {
-        %args_common,
-        %argopt_quiet,
+        %App::BrowserUtils::args_common,
+        %App::BrowserUtils::argopt_quiet,
     },
 };
 sub vivaldi_is_paused {
-    _do_vivaldi('is_paused', @_);
+    App::BrowserUtils::_do_browser('is_paused', 'vivaldi', @_);
 }
 
 $SPEC{terminate_vivaldi} = {
     v => 1.1,
     summary => "Terminate  (kill -KILL) Vivaldi",
     args => {
-        %args_common,
+        %App::BrowserUtils::args_common,
     },
 };
 sub terminate_vivaldi {
-    _do_vivaldi('terminate', @_);
+    App::BrowserUtils::_do_browser('terminate', 'vivaldi', @_);
 }
 
 1;
@@ -167,7 +95,7 @@ App::VivaldiUtils - Utilities related to the Vivaldi browser
 
 =head1 VERSION
 
-This document describes version 0.001 of App::VivaldiUtils (from Perl distribution App-VivaldiUtils), released on 2019-12-05.
+This document describes version 0.002 of App::VivaldiUtils (from Perl distribution App-VivaldiUtils), released on 2019-12-09.
 
 =head1 SYNOPSIS
 
@@ -210,7 +138,7 @@ Arguments ('*' denotes required arguments):
 
 =item * B<users> => I<array[unix::local_uid]>
 
-Kill Vivaldi processes of certain users only.
+Kill browser processes that belong to certain user(s) only.
 
 =back
 
@@ -243,7 +171,7 @@ Arguments ('*' denotes required arguments):
 
 =item * B<users> => I<array[unix::local_uid]>
 
-Kill Vivaldi processes of certain users only.
+Kill browser processes that belong to certain user(s) only.
 
 =back
 
@@ -276,7 +204,7 @@ Arguments ('*' denotes required arguments):
 
 =item * B<users> => I<array[unix::local_uid]>
 
-Kill Vivaldi processes of certain users only.
+Kill browser processes that belong to certain user(s) only.
 
 =back
 
@@ -309,7 +237,7 @@ Arguments ('*' denotes required arguments):
 
 =item * B<users> => I<array[unix::local_uid]>
 
-Kill Vivaldi processes of certain users only.
+Kill browser processes that belong to certain user(s) only.
 
 =back
 
@@ -346,7 +274,7 @@ Arguments ('*' denotes required arguments):
 
 =item * B<users> => I<array[unix::local_uid]>
 
-Kill Vivaldi processes of certain users only.
+Kill browser processes that belong to certain user(s) only.
 
 =back
 
@@ -388,6 +316,8 @@ L<App::FirefoxUtils>
 
 L<App::OperaUtils>
 
+L<App::BrowserUtils>
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
